(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.tsProve = {}));
}(this, (function (exports) { 'use strict';

  /**
   * Type guard utility with guards set for primitives
   */
  var is = function (fn) { return function (value) { return fn(value); }; };
  is.null = is(function (y) { return y === null; });
  is.array = is(Array.isArray);
  is.string = is(function (value) { return typeof value === 'string'; });
  is.number = is(function (value) { return typeof value === 'number'; });
  is.symbol = is(function (value) { return typeof value === 'symbol'; });
  is.boolean = is(function (value) { return value === true || value === false; });
  is.function = is(function (value) { return typeof value === 'function'; });
  is.undefined = is(function (y) { return y === undefined; });
  is.shape = is(function (value) { return typeof value === 'object' && !is.function(value) && !is.array(value) && value !== null; });
  /**
   * Proof utilities
   */
  var success = function (x) { return [null, x]; };
  var failure = function (err, x) { return [err, x]; };
  var isResolved = function (x) {
      return is(function (y) { return !!(is.array(y) && (is.null(y[0]) || is.string(y[0]))); })(x);
  };
  var isProved = function (x) {
      return is(function (y) { return !!(is.array(y) && y[0] === null); })(x);
  };
  var check = function (prf) { return function (y) {
      if (is.function(y))
          throw new Error("recieved callback {" + y.toString() + "} expected none function value");
      return prf(y)[0] || true;
  }; };
  var result = function (value, result) {
      return result !== true ? failure(result, value) : success(value);
  };
  /**
   * Generate output string for all javascript types
   */
  var outputString = function (val) {
      if (is.undefined(val))
          return 'undefined';
      if (is.null(val))
          return 'null';
      if (is.shape(val))
          return JSON.stringify(val, function (_key, value) { return (is.string(value) ? value.replace(/\n/gim, "\n  ") : value); }, 2)
              .replace(/"/gim, '')
              .replace(/\\n/gim, "\n");
      return val.toString();
  };

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  function __spreadArrays() {
      for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
      for (var r = Array(s), k = 0, i = 0; i < il; i++)
          for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
              r[k] = a[j];
      return r;
  }

  /**
   * Where the magic happens
   */
  var prove = function (valueOrCheck, trs) {
      if (trs === void 0) { trs = []; }
      return is(is.function)(valueOrCheck)
          ? function (y) { return prove(y, __spreadArrays(trs, [valueOrCheck])); }
          : result(valueOrCheck, trs.reduce(function (a, b) { return (a !== true ? a : b(valueOrCheck)); }, true));
  };
  /**
   * Proof for javasript primitives
   */
  prove.string = prove(function (x) { return is.string(x) || 'Expected string'; });
  prove.number = prove(function (x) { return is.number(x) || 'Expected number'; });
  prove.boolean = prove(function (x) { return is.boolean(x) || 'Expected boolean'; });
  prove.symbol = prove(function (x) { return is.symbol(x) || 'Expected symbol'; });
  prove.null = prove(function (x) { return is.null(x) || 'Expected null'; });
  prove.undefined = prove(function (x) { return is.undefined(x) || 'Expected undefined'; });
  /**
   * Structured key value Proof
   */
  prove.shape = function (shp) {
      return prove(function (x) {
          return Object.keys(shp).reduce(function (a, b) {
              var _a;
              if (is.string(a))
                  return a;
              var res = shp[b](x[b]);
              if (isResolved(res) && !isProved(res))
                  return outputString(__assign(__assign({}, a[1]), (_a = {}, _a[b] = res[0], _a)));
              return true;
          }, true);
      });
  };
  prove.optional = function (prf) {
      return prove(function (x) { return is.undefined(x) || check(prf)(x); });
  };
  /**s
   * Structured array Proof
   */
  prove.array = function (arrayOf) {
      return prove(function (x) { return is.array(x) || 'Expected array'; })(function (y) {
          return y.reduce(function (a, b, i) {
              if (is.string(a))
                  return a;
              var res = arrayOf(b);
              if (isResolved(res) && !isProved(res))
                  return "[ ([" + i + "] " + res[0] + ") ]";
              return a;
          }, true);
      });
  };
  /**
   * Logical proof which checks that item is one of some type
   */
  prove.or = function () {
      var proofs = [];
      for (var _i = 0; _i < arguments.length; _i++) {
          proofs[_i] = arguments[_i];
      }
      return prove(function (value) {
          return proofs.reduce(function (a, prf) {
              if (a === true)
                  return true;
              var result = check(prf)(value);
              return result === true
                  ? true
                  : (a + (a.length > 0 ? ' or ' : '') + result).replace(/\sExpected/gi, '');
          }, '');
      });
  };
  /**
   * Strict equals
   */
  prove.equals = function (value) {
      return prove(function (input) { return input === value || outputString(value) + " is not equal to " + outputString(input); });
  };

  exports.check = check;
  exports.default = prove;
  exports.failure = failure;
  exports.is = is;
  exports.isProved = isProved;
  exports.outputString = outputString;
  exports.success = success;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
