"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
/**
 * Where the magic happens
 */
var prove = function (valueOrCheck, trs) {
    if (trs === void 0) { trs = []; }
    return utils_1.is(utils_1.is.function)(valueOrCheck)
        ? function (y) { return prove(y, __spreadArrays(trs, [valueOrCheck])); }
        : utils_1.result(valueOrCheck, trs.reduce(function (a, b) { return (a !== true ? a : b(valueOrCheck)); }, true));
};
/**
 * Proof for javasript primitives
 */
prove.string = prove(function (x) { return utils_1.is.string(x) || 'Expected string'; });
prove.number = prove(function (x) { return utils_1.is.number(x) || 'Expected number'; });
prove.boolean = prove(function (x) { return utils_1.is.boolean(x) || 'Expected boolean'; });
prove.symbol = prove(function (x) { return utils_1.is.symbol(x) || 'Expected symbol'; });
prove.null = prove(function (x) { return utils_1.is.null(x) || 'Expected null'; });
prove.undefined = prove(function (x) { return utils_1.is.undefined(x) || 'Expected undefined'; });
/**
 * Structured key value Proof
 */
prove.shape = function (shp) {
    return prove(function (x) {
        return Object.keys(shp).reduce(function (a, b) {
            var _a;
            if (utils_1.is.string(a))
                return a;
            var res = shp[b](x[b]);
            if (utils_1.isResolved(res) && !utils_1.isProved(res))
                return utils_1.outputString(__assign(__assign({}, a[1]), (_a = {}, _a[b] = res[0], _a)));
            return true;
        }, true);
    });
};
prove.optional = function (prf) {
    return prove(function (x) { return utils_1.is.undefined(x) || utils_1.check(prf)(x); });
};
/**s
 * Structured array Proof
 */
prove.array = function (arrayOf) {
    return prove(function (x) { return utils_1.is.array(x) || 'Expected array'; })(function (y) {
        return y.reduce(function (a, b, i) {
            if (utils_1.is.string(a))
                return a;
            var res = arrayOf(b);
            if (utils_1.isResolved(res) && !utils_1.isProved(res))
                return "[ ([" + i + "] " + res[0] + ") ]";
            return a;
        }, true);
    });
};
/**
 * Logical proof which checks that item is one of some type
 */
prove.or = function () {
    var proofs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        proofs[_i] = arguments[_i];
    }
    return prove(function (value) {
        return proofs.reduce(function (a, prf) {
            if (a === true)
                return true;
            var result = utils_1.check(prf)(value);
            return result === true
                ? true
                : (a + (a.length > 0 ? ' or ' : '') + result).replace(/\sExpected/gi, '');
        }, '');
    });
};
/**
 * Strict equals
 */
prove.equals = function (value) {
    return prove(function (input) { return input === value || utils_1.outputString(value) + " is not equal to " + utils_1.outputString(input); });
};
exports.default = prove;
//# sourceMappingURL=prove.js.map