"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Type guard utility with guards set for primitives
 */
exports.is = function (fn) { return function (value) { return fn(value); }; };
exports.is.null = exports.is(function (y) { return y === null; });
exports.is.array = exports.is(Array.isArray);
exports.is.string = exports.is(function (value) { return typeof value === 'string'; });
exports.is.number = exports.is(function (value) { return typeof value === 'number'; });
exports.is.symbol = exports.is(function (value) { return typeof value === 'symbol'; });
exports.is.boolean = exports.is(function (value) { return value === true || value === false; });
exports.is.function = exports.is(function (value) { return typeof value === 'function'; });
exports.is.undefined = exports.is(function (y) { return y === undefined; });
exports.is.shape = exports.is(function (value) { return typeof value === 'object' && !exports.is.function(value) && !exports.is.array(value) && value !== null; });
/**
 * Proof utilities
 */
exports.success = function (x) { return [null, x]; };
exports.failure = function (err, x) { return [err, x]; };
exports.isResolved = function (x) {
    return exports.is(function (y) { return !!(exports.is.array(y) && (exports.is.null(y[0]) || exports.is.string(y[0]))); })(x);
};
exports.isProved = function (x) {
    return exports.is(function (y) { return !!(exports.is.array(y) && y[0] === null); })(x);
};
exports.check = function (prf) { return function (y) {
    if (exports.is.function(y))
        throw new Error("recieved callback {" + y.toString() + "} expected none function value");
    return prf(y)[0] || true;
}; };
exports.result = function (value, result) {
    return result !== true ? exports.failure(result, value) : exports.success(value);
};
/**
 * Generate output string for all javascript types
 */
exports.outputString = function (val) {
    if (exports.is.undefined(val))
        return 'undefined';
    if (exports.is.null(val))
        return 'null';
    if (exports.is.shape(val))
        return JSON.stringify(val, function (_key, value) { return (exports.is.string(value) ? value.replace(/\n/gim, "\n  ") : value); }, 2)
            .replace(/"/gim, '')
            .replace(/\\n/gim, "\n");
    return val.toString();
};
//# sourceMappingURL=utils.js.map