export declare type Error = string;
export declare type True = true;
export declare type Tuple<A, B> = [A, B];
export declare type Function<A extends any[], B extends any> = (...args: A) => B;
export declare type Value = number | string | boolean | symbol | null | undefined | Record<any, any> | any[];
/**
 * Return types
 */
export declare type Valid = True | Error;
export declare type Success<T> = Tuple<null, T>;
export declare type Failure<T = unknown> = Tuple<string, T>;
/**
 * Compose validation functions
 * @param any the input type for the proof
 * @return (Proof | Success | Faillure)
 */
export interface Proof<R extends Value> {
    <T extends Check<R> | R>(x: T, trs?: Check<R>[]): T extends Check<R> ? Proof<R> : Failure | Success<R>;
}
export declare type OptionalProp<T extends Proof<any>> = {
    optional: T;
};
export declare type TypeOfOpional<T extends OptionalProp<any>> = T extends OptionalProp<infer D> ? D : never;
export declare type ProofType<T extends Proof<any>> = T extends Proof<infer D> ? D : never;
export declare type Check<T extends any = unknown> = Function<[T], Valid>;
declare type FixedKeys<T extends Record<any, Proof<any>>> = {
    [Key in keyof T]: ProofType<T[Key]> extends OptionalProp<any> ? never : Key;
}[keyof T];
declare type OptionalKeys<T extends Record<any, Proof<any>>> = {
    [Key in keyof T]: ProofType<T[Key]> extends OptionalProp<any> ? Key : never;
}[keyof T];
declare type CombineOptionals<T extends Record<any, Proof<any>>> = {
    [Key in FixedKeys<T>]: ProofType<T[Key]>;
} & {
    [Key in OptionalKeys<T>]?: ProofType<T[Key]> extends OptionalProp<infer D> ? ProofType<D> : never;
};
declare type Construct<T extends Record<any, any>> = {
    [Key in keyof T]: T[Key];
};
export declare type Shape<T extends Record<any, Proof<any>>> = Construct<CombineOptionals<T>>;
export {};
